// editConfig.js

var configError = false;

var networkCommandOptions = new Array;
networkCommandOptions['ip'] = 0;
networkCommandOptions['ifconfig'] = 1;

var logLevelOptions = new Array;
logLevelOptions['debug'] = 0;
logLevelOptions['info'] = 1;
logLevelOptions['notice'] = 2;
logLevelOptions['warning'] = 3;
logLevelOptions['err'] = 4;
logLevelOptions['crit'] = 5;
logLevelOptions['alert'] = 6;
logLevelOptions['emerg'] = 7;

var rateLimitOptions = new Array;
rateLimitOptions['second'] = 0;
rateLimitOptions['s'] = 0;
rateLimitOptions['minute'] = 1;
rateLimitOptions['m'] = 1;
rateLimitOptions['hour'] = 2;
rateLimitOptions['h'] = 2;
rateLimitOptions['day'] = 3;
rateLimitOptions['d'] = 3;

var kernelTypeOptions = new Array;
kernelTypeOptions['modular'] = 0;
kernelTypeOptions['monolithic'] = 1;

function networkCommandSelectOptions()
{
  var output = '';
  for (var option in networkCommandOptions)
  {
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

function logLevelSelectOptions()
{
  var output = '';
  for (var option in logLevelOptions)
  {
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

function rateLimitSelectOptions()
{
  var output = '';
  for (var option in rateLimitOptions)
  {
    if (option.length == 1)
    {
      continue;
    }
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

function kernelTypeSelectOptions()
{
  var output = '';
  for (var option in kernelTypeOptions)
  {
    output += '    <option value="' + option + '">' + option + '</option>\n';
  }

  return output;
}

function toggleModularSettings()
{
  var f = document.configF;

  if (f.modular.selectedIndex == 0)
  {
    // modular
    f.ftp.disabled = false;
    f.irc.disabled = false;
    toggleFTPParams();
    toggleIRCParams();
  }
  else
  {
    // monolithic
    f.ftp.disabled = true;
    f.ftpParams.disabled = true;
    f.irc.disabled = true;
    f.ircParams.disabled = true;
  }
}

function toggleFTPParams()
{
  var f = document.configF;

  if (f.modular.selectedIndex == 0)
  {
    f.ftpParams.disabled = (f.ftp.selectedIndex == 1 ? true : false);
  }
}

function toggleIRCParams()
{
  var f = document.configF;

  if (f.modular.selectedIndex == 0)
  {
    f.ircParams.disabled = (f.irc.selectedIndex == 1 ? true : false);
  }
}

function setLimitItem(name, value)
{
  var f = document.configF;
  var textField = name + 'Value';
  var selectField = name + 'Options';

  // figure out if the value is a limit or 0.
  if (value == '0')
  {
    eval('f.' + textField + '.value = value;');
    eval('f.' + selectField + '.selectedIndex = -1;');
  }
  else
  {
    // split the value on the /
    var parts = value.match(/^(.+)\/(.+)$/);
    eval('f.' + textField + '.value = parts[1];');
    eval('f.' + selectField + '.selectedIndex = rateLimitOptions[parts[2]];');
  }
}

function validateLimitItem(name, value)
{
  var f = document.configF;
  var textField = name + 'Value';
  var selectField = name + 'Options';
  var textValue = "";
  var selectIndex = -1;

  eval('textValue = f.' + textField + '.value;');
  eval('selectIndex = f.' + selectField + '.selectedIndex;');

  var digits = /^(\d+)$/;
  if (!digits.test(textValue))
  {
    alert(name + " = '" + textValue + "' is invalid!");
    configError = true;
    return value;
  }
  if (textValue == 0)
  {
    if (textValue != value)
    {
      entryChangesMade = true;
      value = textValue;
    }
  }
  else
  {
    var selectValue = "";
    if (selectIndex >= 0)
    {
      eval('selectValue = f.' + selectField + '.options[' + selectIndex + '].value;');
    }
    else
    {
      alert("You must select the rate limit modifier for " + name + "!");
      configError = true;
      return value;
    }

    if (value == 0)
    {
      entryChangesMade = true;
      value = textValue + '/' + selectValue;
    }
    else
    {
      // split the value on the /
      var parts = value.match(/^(.+)\/(.+)$/);
      if (parts[1] != textValue)
      {
        entryChangesMade = true;
        value = textValue + '/' + selectValue;
      }
      else if (parts[2] != selectValue)
      {
        if (parts[2].length == 1 && selectValue.charAt(0) == parts[2])
        {
          // do nothing since this is just the case of m => minutes, etc.
        }
        else
        {
          entryChangesMade = true;
          value = textValue + '/' + selectValue;
        }
      }
    }
  }

  return value;
}

// define the <config> arguments
nameLookups['config'] = new Array;

createLookupEntry('config', 'networkCommand', 'Network Command', 'select', networkCommandSelectOptions(), 'Specify what program should be used to determine network settings (IP Addresses, BroadCast Addresses, etc.)');
createLookupEntry('config', 'mangleRules', 'Mangle Rules Enabled', 'select', trueFalseSelectOptions(), 'If true, the tos (Type Of Service) will be set for certain common protocols.  This is not really suggested to be done.');
createLookupEntry('config', 'ecn', 'ECN Enabled', 'select', trueFalseSelectOptions(), 'If true, then ECN (Explicit Congestion Notification) is turned on in the kernel.  This can cause problems with older routers that are not ECN aware and can cause traffic to be dropped.');
createLookupEntry('config', 'bridgeSupport', 'Bridge Support Enabled', 'select', trueFalseSelectOptions(), 'Do <b>NOT</b> enable this unless you know you need it!<br><br>If true, then Bridge mode support is enabled.  This means that no outbound traffic is NAT\'ed, and the FORWARD chain is enabled even if you only define the external zone.  This support was implemented for a bridge running snort-inline.');
createLookupEntry('config', 'snortInlineSupport', 'Snort-Inline Support Enabled', 'select', trueFalseSelectOptions(), 'Do <b>NOT</b> enable this unless you know you need it!<br><br>If true, then all ESTABLISHED, RELATED packets are QUEUEd to user space for snort-inline to process.  This support was implemented for a bridge running snort-inline.');
createLookupEntry('config', 'dynamicInterfacesMode', 'Dynamic Interfaces Mode', 'select', dynamicInterfacesModeSelectOptions(), 'Specify how you want dynamic interfaces to be worked with.  If you select <i>lookupIP</i>, then the old code that caused the interfaces IP address to be looked up and used will be generated.  If you select <i>ignoreIP</i>, then the interface will not look up the IP address and no rules will be generated that are requiring the interfaces IP address.  This allows laptops to move between networks without having to restart the firewall everytime.');
createLookupEntry('config', 'tcpFlags', 'TCP Flags', 'select', trueFalseSelectOptions(), 'If true, then tcp flags are checked for validity.');
createLookupEntry('config', 'unclean', 'Unclean', 'select', trueFalseSelectOptions(), 'If true, then we use the unclean module to check for invalid packets.  <b>This has been removed in the 2.6 series kernels, so you want to say <i>false</i> if using a 2.6 kernel.</b>');
createLookupEntry('config', 'modular', 'Kernel Type', 'select', kernelTypeSelectOptions(), 'Specify if the iptables code is compiled into the kernel (monolithic) or if it is compiled as modules (modular).', 'onChange="toggleModularSettings();"');
createLookupEntry('config', 'ftp', 'ftp Enabled', 'select', trueFalseSelectOptions(), 'If true, we load the ftp connection tracking modules.', 'onChange="toggleFTPParams();"');
createLookupEntry('config', 'ftpParams', 'ftp Params', 'text', '', 'Specify the ftp connection tracking modules parameters.');
createLookupEntry('config', 'irc', 'irc Enabled', 'select', trueFalseSelectOptions(), 'If true, we load the irc connection tracking modules.', 'onChange="toggleIRCParams();"');
createLookupEntry('config', 'ircParams', 'irc Params', 'text', '', 'Specify the irc connection tracking modules parameters.');
createLookupEntry('config', 'internal', 'internal Zone Enabled', 'select', trueFalseSelectOptions(), 'If true, then the &lt;zone type="internal"&gt; will be enabled and you can create rules using it.');
createLookupEntry('config', 'ipsec', 'ipsec Zone Enabled', 'select', trueFalseSelectOptions(), 'If true, then the &lt;zone type="ipsec"&gt; will be enabled and you can create rules using it.');
createLookupEntry('config', 'dmz', 'dmz Zone Enabled', 'select', trueFalseSelectOptions(), 'If true, then the &lt;zone type="dmz"&gt; will be enabled and you can create rules using it.');
createLookupEntry('config', 'dialin', 'dialin Zone Enabled', 'select', trueFalseSelectOptions(), 'If true, then the &lt;zone type="dialin"&gt; will be enabled and you can create rules using it.');
createLookupEntry('config', 'prefix', 'Log Prefix', 'text', '', 'Specify the string you want all syslog entries prefixed with.  Defaults to "FW: ".');
createLookupEntry('config', 'level', 'Log Level', 'select', logLevelSelectOptions(), 'Specify the level you want all syslog entries logged at.  Defaults to "warning" though "debug" may be better to use.');
createLookupEntry('config', 'taint', 'Tainted Packets Log Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that Tainted Packets should be logged at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');
createLookupEntry('config', 'reserved', 'Reserved Packets Log Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that Reserved Packets (not allowed to traverse the Internet) should be logged at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');
createLookupEntry('config', 'policy', 'Default Policy Packets Log Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that Dropped Packets due to Policy decisions should be logged at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');
createLookupEntry('config', 'reject', 'Reject Packets Log Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that Rejected Packets should be logged at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');
createLookupEntry('config', 'normal', 'Normal Packets Log Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that Accepted Packets should be logged at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');
createLookupEntry('config', 'icmp', 'ICMP Packets Allow Rate', 'mixed', rateLimitSelectOptions(), 'Specify the rate that ICMP Packets should be allowed at (max per period).  If you do not want to limit, then use 0 for the value.  The period does not matter in that case.');

function displayConfigEntry(name, prefixString, rowSpan)
{
  var output = '';
  try {
    var info = getLookupEntry('config', name);
  }
  catch (e) {
    return 'Error getting info! ' + e.message;
  }
  var handler = '';
  var result = info.handler.match(/^(.*onChange=".+)(")(.*)$/);
  if (result != null)
  {
    handler = result[1] + '; entryChangesMade=true;"' + result[3];
  }
  else
  {
    handler = 'onChange="entryChangesMade=true;"';
  }

  // create a table to hold the name and user interaction parts in.
  output += '    <td width="50%"' + (arguments.length == 3 ? ' rowspan="' + rowSpan + '"' : '') + '>\n';  // start the td we are in.
  if (arguments.length >= 2)
  {
    if (prefixString.length > 0)
    {
      output += '      <center><b>' + prefixString + '</b></center>\n';
    }
  }
  output += '      <table width="100%" border="0" cellpadding="0" cellspacing="2">\n';
  output += '        <tr>\n';
  output += '          <td align="right" width="50%"><a href="#" onClick="displayHelp(\'config\', \'' + name + '\'); return false;">' + info.displayString + '</a>&nbsp;</td>\n';
  output += '          <td align="left" width="50%">';

  if (info.type == "select")
  {
    output += '            <select name="' + name + '"' + handler + '>\n';
    output += info.options;
    output += '            </select>\n';
  }
  if (info.type == "text")
  {
    output += '            <input type="text" name="' + name + '" value=""' + handler + ' onKeyPress="entryChangesMade=true;">\n';
  }
  if (info.type == "mixed")
  {
    // display the text field first
    output += '            <input type="text" name="' + name + 'Value" value="" size="4"' + handler + ' onKeyPress="entryChangesMade=true;">';

    // then display the /
    output += ' / ';

    // then display the select box
    output += '            <select name="' + name + 'Options" onChange="entryChangesMade=true;">\n';
    output += info.options;
    output += '            </select>\n';
  }

  output += '          </td>\n';
  output += '        </tr>\n';
  output += '      </table>\n';
  output += '    </td>\n';  // close the td we are in.

  return output;
}

function displayConfig()
{
  var content = getContent();
  var configObj = dataObj.config;

  if (! verifyChangesSaved())
  {
    return;
  }
  entryChangesMade = false;

  displayStatus("Editing Config options...", "info");

  // build up the form html to display the dataObj.configObj info in.
  var html = "";
  html += '  <center><b>Config Options</b></center><br>\n';
  html += '  <table border="0" width="100%" cellpadding="0" cellspacing="2">\n';
  html += '    <tr>\n';
  html += displayConfigEntry('networkCommand');
  html += displayConfigEntry('mangleRules');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('tcpFlags', 'Validity Check:');
  html += displayConfigEntry('ecn');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('unclean');
  html += displayConfigEntry('modular');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('internal', 'Zones:');
  html += displayConfigEntry('ftp', 'Special Protocol Modules:');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('ipsec');
  html += displayConfigEntry('ftpParams');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('dmz');
  html += displayConfigEntry('irc');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('dialin');
  html += displayConfigEntry('ircParams');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += '      <td>&nbsp;</td>\n';
  html += displayConfigEntry('taint', 'Rate Limit:');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('bridgeSupport');
  html += displayConfigEntry('reserved');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('snortInlineSupport');
  html += displayConfigEntry('policy');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('prefix', 'Logging:');
  html += displayConfigEntry('reject');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('level', '');
  html += displayConfigEntry('normal');
  html += '    </tr>\n';
  html += '    <tr>\n';
  html += displayConfigEntry('dynamicInterfacesMode', 'Dynamic Interfaces:');
  html += displayConfigEntry('icmp');
  html += '    </tr>\n';
  html += '  </table>\n';
  html += '   <br>\n';
  html += '  <center><input type="button" name="update" value="Update" onClick="updateConfig();"></center>\n';

  content.innerHTML = html;

  var config = document.configF;

  // set the defaults based upon the current values of dataObj.configObj.
  config.networkCommand.selectedIndex = networkCommandOptions[configObj.networkCommand];
  config.mangleRules.selectedIndex = boolSelectIndex(configObj.mangleRules);
  config.ecn.selectedIndex = boolSelectIndex(configObj.ecn);
  config.bridgeSupport.selectedIndex = boolSelectIndex(configObj.bridgeSupport);
  config.snortInlineSupport.selectedIndex = boolSelectIndex(configObj.snortInlineSupport);
  config.dynamicInterfacesMode.selectedIndex = dynamicInterfacesSelectIndex(configObj.dynamicInterfacesMode);
  config.tcpFlags.selectedIndex = boolSelectIndex(configObj.validityChecks.tcpFlags);
  config.unclean.selectedIndex = boolSelectIndex(configObj.validityChecks.unclean);
  config.internal.selectedIndex = boolSelectIndex(configObj.zones['internal']);
  config.ipsec.selectedIndex = boolSelectIndex(configObj.zones['ipsec']);
  config.dmz.selectedIndex = boolSelectIndex(configObj.zones['dmz']);
  config.dialin.selectedIndex = boolSelectIndex(configObj.zones['dialin']);
  config.modular.selectedIndex = boolSelectIndex(configObj.moduleSupport.modular);
  config.ftp.selectedIndex = boolSelectIndex(configObj.moduleSupport.conntrackers['ftp'].active);
  config.ftpParams.value = configObj.moduleSupport.conntrackers['ftp'].params;
  toggleFTPParams();
  config.irc.selectedIndex = boolSelectIndex(configObj.moduleSupport.conntrackers['irc'].active);
  config.ircParams.value = configObj.moduleSupport.conntrackers['irc'].params;
  toggleIRCParams();
  toggleModularSettings();
  config.prefix.value = configObj.logInfo['prefix'];
  config.level.selectedIndex = logLevelOptions[configObj.logInfo['level']];
  setLimitItem('taint', configObj.limitRates['log'].taint);
  setLimitItem('reserved', configObj.limitRates['log'].reserved);
  setLimitItem('policy', configObj.limitRates['log'].policy);
  setLimitItem('reject', configObj.limitRates['log'].reject);
  setLimitItem('normal', configObj.limitRates['log'].normal);
  setLimitItem('icmp', configObj.limitRates['icmp']);
}

function updateConfig()
{
  var config = document.configF;
  var configObj = dataObj.config;

  if (config.networkCommand.selectedIndex != networkCommandOptions[configObj.networkCommand])
  {
    entryChangesMade = true;
    configObj.networkCommand = config.networkCommand.options[config.networkCommand.selectedIndex].value;
  }
  if (isTrueSelect(config.mangleRules.selectedIndex) != configObj.mangleRules)
  {
    entryChangesMade = true;
    configObj.mangleRules = isTrueSelect(config.mangleRules.selectedIndex);
  }
  if (isTrueSelect(config.ecn.selectedIndex) != configObj.ecn)
  {
    entryChangesMade = true;
    configObj.ecn = isTrueSelect(config.ecn.selectedIndex);
  }
  if (isTrueSelect(config.bridgeSupport.selectedIndex) != configObj.bridgeSupport)
  {
    entryChangesMade = true;
    configObj.bridgeSupport = isTrueSelect(config.bridgeSupport.selectedIndex);
  }
  if (isTrueSelect(config.snortInlineSupport.selectedIndex) != configObj.snortInlineSupport)
  {
    entryChangesMade = true;
    configObj.snortInlineSupport = isTrueSelect(config.snortInlineSupport.selectedIndex);
  }
  if (config.dynamicInterfacesMode.options[config.dynamicInterfacesMode.selectedIndex].value != configObj.dynamicInterfacesMode)
  {
    entryChangesMade = true;
    configObj.dynamicInterfacesMode = config.dynamicInterfacesMode.options[config.dynamicInterfacesMode.selectedIndex].value;
  }
  if (isTrueSelect(config.tcpFlags.selectedIndex) != configObj.validityChecks.tcpFlags)
  {
    entryChangesMade = true;
    configObj.validityChecks.tcpFlags = isTrueSelect(config.tcpFlags.selectedIndex);
  }
  if (isTrueSelect(config.unclean.selectedIndex) != configObj.validityChecks.unclean)
  {
    entryChangesMade = true;
    configObj.validityChecks.unclean = isTrueSelect(config.unclean.selectedIndex);
  }
  if (isTrueSelect(config.ipsec.selectedIndex) != configObj.zones['ipsec'])
  {
    entryChangesMade = true;
    configObj.zones['ipsec'] = isTrueSelect(config.ipsec.selectedIndex);
  }
  if (isTrueSelect(config.internal.selectedIndex) != configObj.zones['internal'])
  {
    // make sure we are not disabling a boundTo interface
    if (configObj.zones['internal'] && ipsecUsingZone('internal'))
    {
      alert("Disabling Zone 'internal' will break IPSec support!");
      configError = true;
    }
    else
    {
      entryChangesMade = true;
      configObj.zones['internal'] = isTrueSelect(config.internal.selectedIndex);
    }
  }
  if (isTrueSelect(config.dmz.selectedIndex) != configObj.zones['dmz'])
  {
    if (configObj.zones['dmz'] && ipsecUsingZone('dmz'))
    {
      alert("Disabling Zone 'dmz' will break IPSec support!");
      configError = true;
    }
    else
    {
      entryChangesMade = true;
      configObj.zones['dmz'] = isTrueSelect(config.dmz.selectedIndex);
    }
  }
  if (isTrueSelect(config.dialin.selectedIndex) != configObj.zones['dialin'])
  {
    if (configObj.zones['dialin'] && ipsecUsingZone('dialin'))
    {
      alert("Disabling Zone 'dialin' will break IPSec support!");
      configError = true;
    }
    else
    {
      entryChangesMade = true;
      configObj.zones['dialin'] = isTrueSelect(config.dialin.selectedIndex);
    }
  }
  if (isTrueSelect(config.modular.selectedIndex) != configObj.moduleSupport.modular)
  {
    entryChangesMade = true;
    configObj.moduleSupport.modular = isTrueSelect(config.modular.selectedIndex);
  }
  if (isTrueSelect(config.ftp.selectedIndex) != configObj.moduleSupport.conntrackers['ftp'].active)
  {
    entryChangesMade = true;
    configObj.moduleSupport.conntrackers['ftp'].active = isTrueSelect(config.ftp.selectedIndex);
  }
  if (config.ftpParams.value != configObj.moduleSupport.conntrackers['ftp'].params)
  {
    entryChangesMade = true;
    configObj.moduleSupport.conntrackers['ftp'].params = config.ftpParams.value;
  }
  if (isTrueSelect(config.irc.selectedIndex) != configObj.moduleSupport.conntrackers['irc'].active)
  {
    entryChangesMade = true;
    configObj.moduleSupport.conntrackers['irc'].active = isTrueSelect(config.irc.selectedIndex);
  }
  if (config.ircParams.value != configObj.moduleSupport.conntrackers['irc'].params)
  {
    entryChangesMade = true;
    configObj.moduleSupport.conntrackers['irc'].params = config.ircParams.value;
  }
  if (config.prefix.value != configObj.logInfo['prefix'])
  {
    entryChangesMade = true;
    configObj.logInfo['prefix'] = config.prefix.value;
  }
  if (config.level.selectedIndex != logLevelOptions[configObj.logInfo['level']])
  {
    entryChangesMade = true;
    configObj.logInfo['level'] = config.level.options[config.level.selectedIndex].value;
  }
  configObj.limitRates['log'].taint = validateLimitItem('taint', configObj.limitRates['log'].taint);
  configObj.limitRates['log'].reserved = validateLimitItem('reserved', configObj.limitRates['log'].reserved);
  configObj.limitRates['log'].policy = validateLimitItem('policy', configObj.limitRates['log'].policy);
  configObj.limitRates['log'].reject = validateLimitItem('reject', configObj.limitRates['log'].reject);
  configObj.limitRates['log'].normal = validateLimitItem('normal', configObj.limitRates['log'].normal);
  configObj.limitRates['icmp'] = validateLimitItem('icmp', configObj.limitRates['icmp']);

  if (configError)
  {
    // reset the error flag.
    configError = false;
  }
  else
  {
    if (entryChangesMade)
    {
      changesMade = true;
      entryChangesMade = false;
    }
    redisplayEditScreen();
  }
}

function ipsecUsingZone(zone)
{
  if (zone == 'ipsec')
  {
    alert("ipsecUsingZone(ipsec) is invalid!");
    return 0;
  }
  if (dataObj.zones[zone] == null)
  {
    alert("ipsecUsingZone(" + zone + ") is invalid!");
  }

  // first make sure ipsec is enabled.
  if (dataObj.config.zones['ipsec'])
  {
    // now loop over the interfaces defined in ipsec and see if any are boundTo in the specified zone.
    for (var alias in dataObj.zones['ipsec'])
    {
      for (var zAlias in dataObj.zones[zone])
      {
        if (zAlias == dataObj.zones['ipsec'][alias].boundTo)
        {
          return 1;
        }
      }
    }
  }

  return 0;
}
